@@ -2,13 +2,14 @@
 import asyncio
 import logging
 from typing import Optional
+from datetime import datetime
 
 from aiohttp import ClientSession, ClientTimeout
 from aiohttp.client_exceptions import ClientError
 
 from pysmartweatherio.errors import InvalidApiKey, RequestError, ResultError
 from pysmartweatherio.helper_functions import ConversionFunctions
-from pysmartweatherio.dataclasses import StationData
+from pysmartweatherio.dataclasses import StationData, ForecastDataDaily, ForecastDataHourly
 from pysmartweatherio.const import (
     BASE_URL,
     DEFAULT_TIMEOUT,
@@ -16,6 +17,7 @@ from pysmartweatherio.const import (
     UNIT_SYSTEM_IMPERIAL,
     UNIT_TEMP_CELCIUS,
     UNIT_TEMP_FAHRENHEIT,
+    UNIT_PRESSURE_MB,
     UNIT_PRESSURE_HPA,
     UNIT_PRESSURE_INHG,
     UNIT_PRECIP_IN,
@@ -30,6 +32,8 @@ from pysmartweatherio.const import (
     UNIT_TYPE_RAIN,
     UNIT_TYPE_PRESSURE,
     UNIT_TYPE_DISTANCE,
+    FORECAST_TYPE_DAILY,
+    FORECAST_TYPE_HOURLY,
 )
 
 _LOGGER = logging.getLogger(__name__)
@@ -52,6 +56,8 @@ class SmartWeather:
         self._to_wind_unit = to_wind_unit
         self._session: ClientSession = session
         self.req = session
+        self._latitude = None
+        self._longitude = None
 
         if self._to_units == UNIT_SYSTEM_METRIC:
             self._to_units_temp = UNIT_TEMP_CELCIUS
@@ -69,49 +75,99 @@ class SmartWeather:
             self._to_units_distance = UNIT_DISTANCE_MI
 
     async def get_station_name(self) -> None:
+        """Returns the Station Name."""
         return await self._station_name_by_station_id()
 
     async def get_station_data(self) -> None:
+        """Returns current sensor data."""
         return await self._current_station_data()
 
+    async def get_station_hardware(self) -> None:
+        """Returns station hardware data."""
+        return await self._station_information()
+
+    async def get_forecast(self, forecast_type=FORECAST_TYPE_DAILY, hours_to_show=24) -> None:
+        """Returns station Weather Forecast."""
+        return await self._forecast_data(forecast_type, hours_to_show)
+
     async def get_units(self) -> None:
         """Returns the units used for Values."""
+        if self._to_units == UNIT_SYSTEM_METRIC:
+            unit_temp = "°C"
+            unit_wind = "m/s"
+            if self._to_units_wind == UNIT_WIND_KMH:
+                unit_wind = "km/h"
+            unit_rain = "mm"
+            unit_pressure = "hPa"
+            unit_distance = "km"
+        else:
+            unit_temp = "°F"
+            unit_wind = "mi/h"
+            unit_rain = "in"
+            unit_pressure = "inHg"
+            unit_distance = "mi"
+
         units = {
-            UNIT_TYPE_TEMP: self._to_units_temp,
-            UNIT_TYPE_WIND: self._to_units_wind,
-            UNIT_TYPE_RAIN: self._to_units_precip,
-            UNIT_TYPE_PRESSURE: self._to_units_pressure,
-            UNIT_TYPE_DISTANCE: self._to_units_distance,
+            UNIT_TYPE_TEMP: unit_temp,
+            UNIT_TYPE_WIND: unit_wind,
+            UNIT_TYPE_RAIN: unit_rain,
+            UNIT_TYPE_PRESSURE: unit_pressure,
+            UNIT_TYPE_DISTANCE: unit_distance,
         }
         return units
 
+    async def _station_information(self) -> None:
+        """Return Information about the station HW."""
+        endpoint = f"stations/{self._station_id}?api_key={self._api_key}"
+        json_data = await self.async_request("get", endpoint)
+        
+        for row in json_data["stations"]:
+            items = {}
+            name = row["name"]
+            self._latitude = row["latitude"]
+            self._longitude = row["longitude"]
+            for item in row["devices"]:
+                if "device_type" in item:
+                    if item["device_type"] == "HB":
+                        items = {
+                            "station_name": name,
+                            "latitude": self._latitude,
+                            "longitude": self._longitude,
+                            "station_type": "AIR & SKY",
+                            "serial_number": item["serial_number"],
+                            "device_id": item["device_id"],
+                            "firmware_revision": item["firmware_revision"],
+                            "hardware_revision": item["hardware_revision"],
+                        }
+                    if item["device_type"] == "ST":
+                        items = {
+                            "station_name": name,
+                            "latitude": self._latitude,
+                            "longitude": self._longitude,
+                            "station_type": "Tempest",
+                            "serial_number": item["serial_number"],
+                            "device_id": item["device_id"],
+                            "firmware_revision": item["firmware_revision"],
+                            "hardware_revision": item["hardware_revision"],
+                        }
+                        break
+
+            if items:
+                return items
+
+
     async def _station_name_by_station_id(self) -> None:
         """Return Station name from the Station ID."""
-        endpoint = f"station/{self._station_id}?api_key={self._api_key}"
+        endpoint = f"observations/station/{self._station_id}?api_key={self._api_key}"
         json_data = await self.async_request("get", endpoint)
 
         return self._station_id if json_data.get("station_name") is None else json_data.get("station_name")
 
     async def _current_station_data(self) -> None:
         """Return current observation data for the Station."""
-        endpoint = f"station/{self._station_id}?api_key={self._api_key}"
+        endpoint = f"observations/station/{self._station_id}?api_key={self._api_key}"
         json_data = await self.async_request("get", endpoint)
 
-        # if self._to_units == UNIT_SYSTEM_METRIC:
-        #     to_units_temp = UNIT_TEMP_CELCIUS
-        #     to_units_pressure = UNIT_PRESSURE_HPA
-        #     to_units_wind = UNIT_WIND_MS
-        #     if self._to_wind_unit == UNIT_WIND_KMH:
-        #         to_units_wind = UNIT_WIND_KMH
-        #     to_units_precip = UNIT_PRECIP_MM
-        #     to_units_distance = UNIT_DISTANCE_KM
-        # else:
-        #     to_units_temp = UNIT_TEMP_FAHRENHEIT
-        #     to_units_pressure = UNIT_PRESSURE_INHG
-        #     to_units_wind = UNIT_WIND_MPH
-        #     to_units_precip = UNIT_PRECIP_IN
-        #     to_units_distance = UNIT_DISTANCE_MI
-
         row = json_data.get("station_units")
         if row is not None:
             from_units_temp = row["units_temp"]
@@ -122,6 +178,8 @@ class SmartWeather:
             from_units_direction = row["units_direction"]
             from_units_other = row["units_other"]
 
+        station_name = json_data.get("station_name")
+
         cnv = ConversionFunctions()
         items = []
         observations = json_data.get("obs")
@@ -159,6 +217,7 @@ class SmartWeather:
                 "relative_humidity": 0 if "relative_humidity" not in row else row["relative_humidity"],
                 "station_pressure": 0 if "station_pressure" not in row else
                 await cnv.pressure(row["station_pressure"], from_units_pressure, self._to_units_pressure),
+                "station_name": station_name,
                 "solar_radiation": 0 if "solar_radiation" not in row else row["solar_radiation"],
                 "timestamp": None if "timestamp" not in row else
                 await cnv.epoch_to_datetime(row["timestamp"]),
@@ -174,7 +233,68 @@ class SmartWeather:
             items.append(StationData(item))
 
         return items
-            
+
+    async def _forecast_data(self, forecast_type, hours_to_show) -> None:
+        """Return Forecast data for the Station."""
+        if self._latitude is None:
+            # _LOGGER.debug(f"LAT: {self._latitude}")
+            await self._station_information()
+
+        cnv = ConversionFunctions()
+        endpoint = f"better_forecast?station_id={self._station_id}&api_key={self._api_key}&lat={self._latitude}&lon={self._longitude}"
+        json_data = await self.async_request("get", endpoint)
+        items = []
+
+        forecast = json_data.get("forecast")
+        if forecast_type == FORECAST_TYPE_DAILY:
+            for row in forecast[FORECAST_TYPE_DAILY]:
+                item = {
+                    "timestamp": datetime.fromtimestamp(row["day_start_local"]),
+                    "conditions": row["conditions"],
+                    "icon": row["icon"],
+                    "sunrise": datetime.fromtimestamp(row["sunrise"]),
+                    "sunset": datetime.fromtimestamp(row["sunset"]),
+                    "air_temp_high": await cnv.temperature(row["air_temp_high"], UNIT_TEMP_CELCIUS, self._to_units_temp),
+                    "air_temp_low": await cnv.temperature(row["air_temp_low"], UNIT_TEMP_CELCIUS, self._to_units_temp),
+                    "air_temp_high_color": row["air_temp_high_color"],
+                    "air_temp_low_color": row["air_temp_low_color"],
+                    "precip_probability": row["precip_probability"],
+                    "precip_icon": row["precip_icon"],
+                    "precip_type": row["precip_type"],
+                }
+                items.append(ForecastDataDaily(item))
+        else:
+            cnt = 0
+            for row in forecast[FORECAST_TYPE_HOURLY]:
+                item = {
+                    "timestamp": datetime.fromtimestamp(row["time"]),
+                    "conditions": row["conditions"],
+                    "icon": row["icon"],
+                    "air_temperature": await cnv.temperature(row["air_temperature"], UNIT_TEMP_CELCIUS, self._to_units_temp),
+                    "sea_level_pressure": await cnv.pressure(row["sea_level_pressure"], UNIT_PRESSURE_MB, self._to_units_pressure),
+                    "relative_humidity": row["relative_humidity"],
+                    "precip": await cnv.precip(row["precip"], UNIT_PRECIP_MM, self._to_units_precip),
+                    "precip_probability": row["precip_probability"],
+                    "precip_icon": row["precip_icon"],
+                    "precip_type": row["precip_type"],
+                    "wind_avg": await cnv.wind(row["wind_avg"], UNIT_WIND_MS, self._to_units_wind),
+                    "wind_gust": await cnv.wind(row["wind_gust"], UNIT_WIND_MS, self._to_units_wind),
+                    "wind_avg_color": row["wind_avg_color"],
+                    "wind_gust_color": row["wind_gust_color"],
+                    "wind_direction": row["wind_direction"],
+                    "wind_direction_cardinal": row["wind_direction_cardinal"],
+                    "wind_direction_icon": row["wind_direction_icon"],
+                    "uv": row["uv"],
+                    "feels_like": await cnv.temperature(row["feels_like"], UNIT_TEMP_CELCIUS, self._to_units_temp),
+                }
+                items.append(ForecastDataHourly(item))
+                # Only take the first 24 hours
+                cnt += 1
+                if cnt >= hours_to_show:
+                    break
+
+        return items
+        
     async def async_request(self, method: str, endpoint: str) -> dict:
         """Make a request against the SmartWeather API."""
 
